
b.openHorizontalhideBox("");
    b.create_master_slider(PARAM("ba"), "bass");
b.closeBox();
b.openHorizontalBox("");

    b.create_small_rackknobr(PARAM("ba"), "bass");
    b.create_small_rackknobr(PARAM("tr"), "treble");

b.closeBox();
