b.openHorizontalhideBox("");
b.create_master_slider(PARAM("Freq"), _("Freq"));
b.closeBox();
b.openVerticalBox("");
{
    b.openHorizontalBox("");
    {
	b.create_small_rackknobr(PARAM("Freq"), _("Freq"));
    }
    b.closeBox();
}
b.closeBox();
