
b.openHorizontalhideBox("");
    b.create_master_slider(PARAM("Drive"), "Drive");
b.closeBox();
b.openHorizontalBox("");
    b.create_small_rackknobr(PARAM("Drive"), "Drive");
    b.create_small_rackknobr(PARAM("Gain"), "Gain");
b.closeBox();
