#include "common.rc"

style "guitarix-paintbox"
{
  GxPaintBox::box-gradient = {
    { 0, 19661, 19661, 19661, 32428 }, # 0, 0.3, 0.3, 0.3, 0.49
    { 65536, 29661, 29661, 29661, 32428 }} # 1, 0.45, 0.45, 0.45, 0.49
	
  GxPaintBox::skin-gradient = {
    { 65536, 4107, 4107, 4107, 52428 }} # 0.2, 0, 0, 0.2, 0.2 
	
  GxPaintBox::rack-gradient = {
    { 65536, 2768, 2768, 2768, 52428 }} # 0.2, 0, 0, 0.2, 0.8 

    GxPaintBox::bevel                  = 0.11
    GxPaintBox::inverse                = 0
    GxPaintBox::alternate-box          = { 0, 0, 60, 0 }

  GxPaintBox::width =600
  GxPaintBox::height =180
  GxPaintBox::icon-set =7
  stock['gxhead'] = {{"live_wallpaper.png"}}
}

class "*GxPaintBox" style "guitarix-paintbox"

style "guitarix_wheels"
{
stock["wheel_back"] = {{"tactile_wheel_101f.png"}}

GxWheel::framecount = 101 # use 0 for auto (assumes square frames)
                            #  and >1 for non squareframe animations
}

class "*GxWheel" style "guitarix_wheels"

style "guitarix_slevel"
{
  stock["levelslider"] = {{"simplelevelslider.png"}}
  GxRegler::show-value = 0
  GxLevelSlider::slider--width = 3
}

widget "*.lmw" style:highest "guitarix_slevel"

style "guitarix_waveview"
{
  GxWaveView::box-gradient = {
    { 16214, 65536, 13107, 0, 32428 }, # 0.4, 1, 0.2, 0, 0.8
	{ 52428, 65536, 53107, 0, 32428 }} # 0.8, 0.2, 1, 0.2, 0.8

  fg[NORMAL] = 	{ 0.6, 0.6, 0.6 } # RGBA: 0.8, 0.8, 0.2, 0.6
  font_name = "sans 6"
}

class "*GxWaveView" style:highest "guitarix_waveview"

style "always-show-value"
{
  GxRegler::show-value = 1
}

widget "*.ui.tuner_reference_pitch" style:highest "always-show-value"
widget "*.amp.out_amp" style:highest "always-show-value"
widget "*.show_always" style:highest "always-show-value"
widget "*.midi_out.channel_?.channel" style:highest "always-show-value"
widget "*.midi_out.channel_?.program" style:highest "always-show-value"
widget "*.gx_distortion.split_low_freq" style:highest "always-show-value"
widget "*.gx_distortion.split_middle_freq" style:highest "always-show-value"
widget "*.gx_distortion.split_high_freq" style:highest "always-show-value"

style "guitarix_playhead"
{
  stock["portdisplay"] = {{"playhead.png"}}
  GxRegler::show-value = 0
}

widget "*.playhead" style:highest "guitarix_playhead"

style "guitarix_small_button"
{
  GtkButton::inner-border = {0, 0, 0, 0}
  GtkButton::default-border = {0, 0, 0, 0}
  GtkButton::focus-line-width = 0
  GtkButton::focus-padding = 0
  GtkButton::interior-focus = 0
  GtkButton::inner-spacing = 0
  GtkButton::child-displacement-x = 0
  GtkButton::child-displacement-y = 0
}

widget "*.smallbutton" style:highest "guitarix_small_button"

style "guitarix_switch"
{
  xthickness = 0
  ythickness = 0
  GtkButton::inner-border = {5, 5, 5, 5}
  GtkButton::default-border = {0, 0, 0, 0}
  GtkButton::focus-line-width = 0
  GtkButton::focus-padding = 0
  GtkButton::interior-focus = 0
  GtkButton::child-displacement-x = 0
  GtkButton::child-displacement-y = 0
  GtkButton::button_relief = GTK_RELIEF_NORMAL
  engine "clearlooks" {
    #disable_focus = TRUE
    style = CLASSIC
  }
}

class "*GxSwitch" style:highest "guitarix_switch"

style "guitarix_switch_no_dim"
{
  GtkButton::inner-border = {0, 0, 0, 0}
}

widget "*.no_dim" style:highest "guitarix_switch_no_dim"

style "gx_effect_switch"
{
    #bg[NORMAL]     = "#7d9c57"
    #bg[ACTIVE]     = "#7e9e59"
    #bg[PRELIGHT]   = "#86a765"
    
    engine "clearlooks" {
        focus_color         = { 0, 0, 0 }
        contrast            = 0.99
        menubarstyle        = 0
        reliefstyle         = 0
        radius              = 1.0
        disable_focus       = TRUE
        style               = INVERTED #CLASSIC GLOSSY GUMMY INVERTED
    }
}

widget "*effect_on_off" style:highest "gx_effect_switch"

style "guitarix_default" {
    
    GtkScrollbar::has_backward_stepper    = 0
    GtkScrollbar::has_forward_stepper     = 0
    GtkScrolledWindow::scrollbar_spacing  = 0
    
    GxPaintBox::bevel                     = 0.11
    GxPaintBox::inverse                   = 0
    GxPaintBox::alternate-box             = { 0, 0, 60, 0 }
    
    GtkButton::child-displacement-x       = 0
    GtkButton::child-displacement-y       = 0
    
    GtkTreeView::even-row-color = "#000000"
    GtkTreeView::odd-row-color  = "#000000"
       
    fg[NORMAL]              = { 0.85, 0.85, 0.85 }
    fg[ACTIVE]              = { 1.0, 1.0, 1.0 }
    fg[PRELIGHT]            = { 1.0, 1.0, 1.0 }
    fg[INSENSITIVE]         = { 0.5, 0.5, 0.5 }
    fg[SELECTED]            = { 0.9, 0.9, 0.9 }
    
    bg[NORMAL]              = { 0.10, 0.10, 0.10 }
    bg[ACTIVE]              = { 0.0, 0.0, 0.0 }
    bg[PRELIGHT]            = { 0.25, 0.25, 0.25 }
    bg[INSENSITIVE]         = { 0.2, 0.2, 0.2 }
    bg[SELECTED]            = { 0.20, 0.20, 0.20 }
    
    text[NORMAL]            = { 0.9, 0.9, 0.9 }
    text[ACTIVE]            = "#ff9000"
    text[PRELIGHT]          = { 1.0, 1.0, 1.0 }
    text[INSENSITIVE]       = { 0.5, 0.5, 0.5 }
    text[SELECTED]          = { 1.0, 1.0, 1.0 }
    
    base[NORMAL]            = { 0.0, 0.0, 0.0 }
    base[ACTIVE]            = { 0.18, 0.18, 0.18 }
    base[PRELIGHT]          = { 0.1, 0.1, 0.1 }
    base[INSENSITIVE]       = { 0.2, 0.2, 0.2 }
    base[SELECTED]          = { 0.8, 0.18, 0.18 }
    
    stock["handle_left"]    = {{"handle_left.png"}}
    stock["handle_right"]   = {{"handle_right.png"}}
    stock["logo"]           = {{"logo.png"}}
    stock["rack_preset"]    = {{"rack_preset.png"}}
    stock["switchit_on"]    = {{"switch_on.png"}}
    stock["switchit_off"]   = {{"switch_off.png"}}
    stock["minitoggle_on"]  = {{"switch_mid_on.png"}}
    stock["minitoggle_off"] = {{"switch_mid_off.png"}}
    stock["hslider"]        = {{"hslider.png"}}
    stock["rack_expand"]    = {{"rack_expand.png"}}
    stock["rack_shrink"]    = {{"rack_shrink.png"}}
    stock["insert"]         = {{"insert.png"}}
    stock["background1"]    = {{"background1.png"}}
    stock["background2"]    = {{"background2.png"}}
    stock["smallknob"]      = {{"knob_small.png"}}
    stock["smallknobr"]     = {{"knob_small_r.png"}}
    stock["midknob"]        = {{"knob_mid.png"}}
    stock["bigknob"]        = {{"knob_big.png"}}
    
    engine "clearlooks" {
        focus_color         = { 0, 0, 0 }
        contrast            = 0.66
        menubarstyle        = 0
        reliefstyle         = 0
        radius              = 1.0
        disable_focus       = TRUE
        style               = INVERTED #CLASSIC GLOSSY GUMMY INVERTED
    }
}
class "*GtkWidget*" style:highest "guitarix_default"

style "guitarix_ampdetails" {
    GxPaintBox::alternate-box = { 0, 0, 0, 0 }
    GxPaintBox::inverse = 0
}

style "guitarix_liveplay" {
    bg[NORMAL]   = { 0.05, 0.05, 0.05 }
}
widget "*LivePlay*" style:highest "guitarix_liveplay"


widget "*ampdetails" style:highest "guitarix_ampdetails"


style "guitarix_amp" {
    GxPaintBox::alternate-box = { 0, 0, 33, 33 }
}
widget "*ampbox" style:highest "guitarix_amp"


style "rack_devices" {
    font_name = "sans 9"
}
class "GxPaintBox*" style:highest "rack_devices"


style "guitarix_scrollbars" {
    
    engine "clearlooks" {
        scrollbar_color = "#333333"
    }
    bg[NORMAL]   = { 0.10, 0.10, 0.10 }
    bg[PRELIGHT] = { 0.3, 0.3, 0.3 }
    bg[ACTIVE]   = { 0.3, 0.3, 0.3 }
    bg[INSENSITIVE] = { 0.2, 0.2, 0.2 }
}
widget_class "*Scrollbar*" style:highest "guitarix_scrollbars"


style "guitarix_buttons" {
    bg[NORMAL]     = { 0.175, 0.175, 0.175 }
   # base[NORMAL]   = { 0.175, 0.175, 0.175 }
   # base[SELECTED] = { 0.2, 0.2, 0.2 }
}
class "GtkButton" style:highest "guitarix_buttons"

style "guitarix_plugin_button" {
    fg[NORMAL] = "#ff9000"
    fg[PRELIGHT] = "#ffffff"
    xthickness     = 8
    ythickness     = 4
}
widget "*rackcontainer*GtkButton*" style:highest "guitarix_plugin_button"

style "guitarix_rack_button" {
    fg[NORMAL] = "#ff9000"
    fg[PRELIGHT] = "#ffffff"
    bg[NORMAL] = "black"
    bg[PRELIGHT] = "black"
}
widget "*effects_toolpalette*GtkButton*" style:highest "guitarix_rack_button"


style "guitarix_regler" {
    
    font_name = "Roboto Condensed Regular 7"
    
    GxRegler::value-border      = { 4, 4, 2, 2 } # border around text
    GxRegler::border-radius     = 6
    GxRegler::bevel             = 0.12
    GxSelector::stepper-spacing = 6              # spacing between icon and text box
    
    xthickness     = 8                           # border left and right
    ythickness     = 4                           # border top and bottom
    
    fg[NORMAL]     = "#ff9000"                   # text
    fg[PRELIGHT]   = "#ffffff"                   # text hover
    bg[NORMAL]     = "#222222"                   # border
    bg[PRELIGHT]   = "#222222"                   # border hover
    base[NORMAL]   = { 0.05, 0.05, 0.05 }        # text background
    base[PRELIGHT] = "#000000"                   # text background hover
    text[NORMAL]   = "#ff9000"                   # arrow
    text[PRELIGHT] = "#ffffff"                   # arrow hover
}
class "*GxRegler*" style:highest "guitarix_regler"


style "guitarix_font" {
    font_name = "Roboto Condensed Regular 8"
    fg[NORMAL] =  "#222222" #"#dadfe1" # "#c9d5b9"  #"#ffffff"
}
widget "*.rack_label" style:highest "guitarix_font"
widget "*Value" style:highest "guitarix_font"
widget "*ampbox*label" style:highest "guitarix_font"
widget "*beffekt_label" style:highest "guitarix_font"
widget "*tab_rack*GtkLabel" style:highest "guitarix_font"
#widget "*rack_effect_label" style:highest "guitarix_font"

style "guitarix_font_inverse" {
    font_name = "Roboto Condensed Regular 8"
    fg[NORMAL] =  "#aaaaaa"
}
widget "*rack_label_inverse" style:highest "guitarix_font_inverse"


#buttons in rack devices to drag them
#widget "*effect_reset" style:highest "guitarix_rack_slider"

style "guitarix_effect_title" {
    font_name = "Roboto Condensed Regular 10"
    fg[NORMAL] = "#222222"
}
widget "*rackcontainer*effect_title" style:highest "guitarix_effect_title"
widget "*drag_widget*effect_title" style:highest "guitarix_effect_title"

style "rack_drag_button" {
font_name = "Roboto Condensed Regular 20"
    xthickness = 11
    ythickness = 0
}
widget "*rack_drag_button" style:highest "rack_drag_button"


style "guitarix_amp_logo" {
    
}
widget "*imageAmpLogo" style:highest "guitarix_amp_logo"


style "guitarix_center_box" {
    #bg[NORMAL] = "#292929"
}
widget "*rackcontainer*" style:highest "guitarix_center_box"

style "guitarix_tunerbox" {
    GxPaintBox::alternate-box = { 0, 0, 42, 0 }
}
widget "*tunerbox" style:highest "guitarix_tunerbox"



### KNOBS ###
style "guitarix_knobs" {
    fg[INSENSITIVE] = "#211006"
}
class "GxKnob" style:highest "guitarix_knobs"

style "guitarix_small_knob" {
    GxKnob::x_center          = -1
    GxKnob::y_center          = -1
    GxKnob::ring_radius       = 15
    GxKnob::ring_width        = 2
    GxKnob::ring_led_size     = 3
    GxKnob::ring_led_distance = 2
    GxKnob::indicator_radius  = 11
    GxKnob::indicator_width   = 2
    GxKnob::indicator_length  = 5
}
widget_class "*GxSmallKnob" style:highest "guitarix_small_knob"

style "guitarix_small_knob_r" {
    GxKnob::x_center          = -1
    GxKnob::y_center          = -1
    GxKnob::ring_radius       = 26
    GxKnob::ring_width        = 3
    GxKnob::ring_led_size     = 3
    GxKnob::ring_led_distance = 2
    GxKnob::indicator_radius  = 12
    GxKnob::indicator_width   = 3
    GxKnob::indicator_length  = 3
}
widget_class "*GxSmallKnobR" style:highest "guitarix_small_knob_r"

style "guitarix_mid_knob" {
    GxKnob::x_center          = -1
    GxKnob::y_center          = -1
    GxKnob::ring_radius       = 35
    GxKnob::ring_width        = 6
    GxKnob::ring_led_size     = 2
    GxKnob::ring_led_distance = 2
    GxKnob::indicator_radius  = 27
    GxKnob::indicator_width   = 2
    GxKnob::indicator_length  = 6
}
widget_class "*GxMidKnob" style:highest "guitarix_mid_knob"

style "guitarix_big_knob" {
    GxKnob::x_center          = -1
    GxKnob::y_center          = -1
    GxKnob::ring_radius       = 50.5
    GxKnob::ring_width        = 3
    GxKnob::ring_led_size     = 0
    GxKnob::ring_led_distance = 0
    GxKnob::indicator_radius  = 46
    GxKnob::indicator_width   = 3
    GxKnob::indicator_length  = 6
}
widget_class "*GxBigKnob" style:highest "guitarix_big_knob"


style "guitarix_fastmeter" {
    
    xthickness = 8
    ythickness = 4
    
    bg[NORMAL]     = "#222222"                   # border
    bg[PRELIGHT]   = "#222222"                   # border hover
    
    GxFastMeter::clr-bottom    = "#b96901"
    GxFastMeter::clr-middle    = "#d4cd00"
    GxFastMeter::clr-top       = "#ff0500"
    GxFastMeter::over          = "#ff1100"
    GxFastMeter::mid-pos       = 0.717
    GxFastMeter::dimen         = 2
    GxFastMeter::led-border    = 1
    GxFastMeter::led-width     = 2
    GxFastMeter::led-height    = 2
    GxFastMeter::border-radius = 6
    GxFastMeter::bevel         = 0.12
}
widget_class "*GxFastMeter" style:highest "guitarix_fastmeter"
widget "*meterframe*" style:highest "guitarix_fastmeter"

style "guitarix_simplemeter" {
    
    xthickness = 0
    ythickness = 1
}
widget "*simplemeter" style:highest "guitarix_simplemeter"


style "guitarix_vslider" {
    GxVSlider::slider-width = 36
}
widget_class "*GxVSlider" style:highest "guitarix_vslider"

style "guitarix_hslider" {
    GxHSlider::slider-width = 36
}
widget_class "*GxHSlider" style:highest "guitarix_hslider"


style "guitarix_meter_scale" {
    xthickness = 8
    ythickness = 2
    GxMeterScale::tick-size = 3
    GxMeterScale::tick-space = 1
}
widget_class "*GxMeterScale" style:highest "guitarix_meter_scale"


style "guitarix_notebook" {
    xthickness = 5
    ythickness = 5
    bg[ACTIVE] = { 0.10, 0.10, 0.10 }
    bg[NORMAL] = { 0.10, 0.10, 0.10 }
    engine "clearlooks" {
        contrast            = 0.75
        radius              = 3.0
        style               = GLOSSY #INVERTED CLASSIC GLOSSY GUMMY
    }
}
widget "*tab_rack" style:highest "guitarix_notebook"

style "guitarix_notebook_buttons" {
    fg[NORMAL] = "#ff9000"
    fg[ACTIVE] = "#555555"
}
widget "*tab_rack*GtkLabel" style:highest "guitarix_notebook_buttons"


style "guitarix_notebook_labels" {
    fg[NORMAL] = "#aaaaaa"
}
widget "*tab_rack*label" style:highest "guitarix_notebook_labels"


style "guitarix_frame" {
    bg[NORMAL]              = "#97BB6D"
    engine "clearlooks" {
        contrast            = 1.2
        radius              = 4.0
        disable_focus       = TRUE
    }
}
class "*GtkFrame" style:highest "guitarix_frame"
class "*GtkHSeparator" style:highest "guitarix_frame"

style "guitarix_vsep" {
    bg[NORMAL]              = "#333333"
}
class "*GtkVSeparator" style:highest "guitarix_vsep"


style "guitarix_frame_inversed" {
    bg[NORMAL]              = "#292727"
    engine "clearlooks" {
        contrast            = 1.9
        radius              = 4.0
        disable_focus       = TRUE
    }
}
widget "*frame_inversed" style:highest "guitarix_frame_inversed"


style "guitarix_iredit"
{
  GxIREdit::shade-alpha = 0.6
  GxIREdit::gain-line-color = { 30463, 39167, 20223, 65535 }
  #GxIREdit::zoom-marker-color
  GxIREdit::no-data-color = { 65535, 36000, 0, 65535 }
  GxIREdit::sample-graph-color = { 65535, 36000, 0, 65535 }
  GxIREdit::sample-graph-color-out = { 65535, 36000, 0, 36000 }
}
class "*GxIREdit" style:highest "guitarix_iredit"



#rackcontainer
    #effects_toolpalette
    #amp_toplevel_box
        #rackcontainer
            #monobox
                #upper_rackbox
                    #monoampcontainer:ampdetails
                #monorackcontainer
                    #[unit name]
                        #rack_unit_center
                            #rack_unit_title_bar
                #stereorackcontainerV
            #stereorackbox
                #stereorackcontainerH
    #preset_scrolledbox
    






#preset_pane
#rack_effects
    #effect_category
        #GtkLabel
    #effect_button
        #rack_effect_label
#rack_left
#rack_right








#style "guitarix_caption"
#{
  #font_name = "sans 8"
#}

#style "guitarix_regler"
#{
  #GxRegler::value-spacing = 2
  #GxRegler::value-border = { 6, 6, 3, 3 }
  #font_name = "sans 7.5"
  #GxRegler::value-color = {
    #{ 0, 20971, 52428, 10485, 52428 }} 
#}

#style "guitarix_value_display"
#{
  #GxRegler::value-spacing = 1
  #GxRegler::value-border = { 1, 1, 2, 2 }
  #fg[NORMAL] = 	{ 0.7, 0.7, 0.7 }
  #fg[ACTIVE] = { 0.2, 0.2, 0.2 }
  #fg[PRELIGHT] = { 1.0, 1.0, 1.0 }
  #fg[INSENSITIVE] = { 0.0, 0.0, 0.0 }
  #fg[SELECTED] = { 0.8, 0.8, 0.8 }
  #font_name = "sans 7.5"
#}

#style "guitarix_switch"
#{
  #xthickness = 0
  #ythickness = 0
  #GtkButton::inner-border = {0, 0, 0, 0}
  #GtkButton::default-border = {0, 0, 0, 0}
  #GtkButton::focus-line-width = 0
  #GtkButton::focus-padding = 0
  #GtkButton::interior-focus = 0
  #GtkButton::child-displacement-x = 0
  #GtkButton::child-displacement-y = 0
  #engine "clearlooks" {
    ##disable_focus = TRUE
    #style = CLASSIC
  #}
#}

#style "guitarix_small_button"
#{
  #GtkButton::inner-border = {0, 0, 0, 0}
  #GtkButton::default-border = {0, 0, 0, 0}
  #GtkButton::focus-line-width = 0
  #GtkButton::focus-padding = 0
  #GtkButton::interior-focus = 0
  #GtkButton::inner-spacing = 0
  #GtkButton::child-displacement-x = 0
  #GtkButton::child-displacement-y = 0
#}

#style "guitarix_selector"
#{
  ##cairo_set_source_rgba(cr, 0.4, 1, 0.2, 0.8)
  #fg[NORMAL] = 	{ 0.32, 0.8, 0.16 }
  #GtkRange::trough-border = 2
  #GtkRange::stepper-size = 8
  #GtkRange::stepper-spacing = 2
  #GxRegler::value-border = { 2, 0, 0, 0 }
  #font_name = "sans 7.5"
  #xthickness = 10
  #ythickness = 1
#}

#style "guitarix_waveview"
#{
  #GxWaveView::box-gradient = {
    #{ 26214, 65536, 13107, 0, 52428 }, # 0.4, 1, 0.2, 0, 0.8
	#{ 52428, 13107, 65536, 13107, 52428 }} # 0.8, 0.2, 1, 0.2, 0.8

  #fg[NORMAL] = 	{ 0.48, 0.48, 0.12 } # RGBA: 0.8, 0.8, 0.2, 0.6
  #font_name = "sans 6"
#}

#style "guitarix_slider"
#{
  #GxRegler::value-border = {0, 0, 0, 0}
#}

#style "guitarix_eqslider"
#{
  #font_name = "sans 6"
#}

#style "guitarix_bold_label"
#{
  #font_name = "sans bold 8"
#}

#style "guitarix_fastmeter"
#{
  #GxFastMeter::clr-bottom = "#003808"
  #GxFastMeter::clr-middle = "#00ff00"
  #GxFastMeter::clr-top    = "#ff0000"
  #GxFastMeter::over       = "#ff0000"
#}

#style "guitarix_iredit"
#{
  #GxIREdit::sample-graph-color-out = { 20000, 20000, 20000, 65535 }
  #GxIREdit::shade-alpha = 0.6
  #GxIREdit::gain-line-color = { 0, 60000, 0, 65535 }
#}
#class "*GxIREdit" style:highest "guitarix_iredit"

#style "guitarix_playhead"
#{
  #stock["portdisplay"] = {{"playhead.png"}}
  #GxRegler::show-value = 0
#}

#style "guitarix_slevel"
#{
  #stock["levelslider"] = {{"simplelevelslider.png"}}
  #GxRegler::show-value = 0
  #GxLevelSlider::slider--width = 3
#}


#widget "*.lmw" style:highest "guitarix_slevel"
#widget "*.playhead" style:highest "guitarix_playhead"
#widget "*.smallbutton" style:highest "guitarix_small_button"
#widget "*.effekt_label" style:highest "guitarix_caption"
#widget "*.filter_box.*.effekt_label" style:highest "guitarix_bold_label"
#class "*GxRegler" style:highest "guitarix_regler"
#class "*GxSwitch" style:highest "guitarix_switch"
#class "*GxWaveView" style:highest "guitarix_waveview"
#class "*GxHSlider" style:highest "guitarix_slider"
#class "*GxEQSlider" style:highest "guitarix_eqslider"
#class "*GxSelector" style:highest "guitarix_selector"
#class "*GxFastMeter" style:highest "guitarix_fastmeter"
#class "*GxIREdit" style:highest "guitarix_iredit"
#class "*GxSimpleValueDisplay" style:highest "guitarix_value_display"


#style "always-show-value"
#{
  #GxRegler::show-value = 1
#}

#widget "*.ui.tuner_reference_pitch" style:highest "always-show-value"
#widget "*.amp.out_amp" style:highest "always-show-value"
#widget "*.show_always" style:highest "always-show-value"
#widget "*.midi_out.channel_?.channel" style:highest "always-show-value"
#widget "*.midi_out.channel_?.program" style:highest "always-show-value"
#widget "*.gx_distortion.split_low_freq" style:highest "always-show-value"
#widget "*.gx_distortion.split_middle_freq" style:highest "always-show-value"
#widget "*.gx_distortion.split_high_freq" style:highest "always-show-value"

#style "justahack"
#{
  #fg[NORMAL] = {1.0, 1.0, 1.0 }
  #fg[PRELIGHT] = {0.2, 1.0, 0.2 }
  #fg[SELECTED] = {1.0, 0.0, 0.0 }
  #fg[INSENSITIVE] = {0.4, 0.1, 0.1 }
#}
#widget "gtkmm__GtkWindow.gtkmm__GtkEventBox.gtkmm__GxPaintBox.gtkmm__GtkLabel" style:highest "justahack"
